@@ -4,7 +4,7 @@ use strict;
 use warnings;
 
 
-our $VERSION = 'v0.40.3';
+our $VERSION = 'v0.41.0';
 
 =head1 NAME
 
@@ -47,7 +47,7 @@ use aliased 'OpenTracing::Implementation::DataDog::Span';
 use aliased 'OpenTracing::Implementation::DataDog::SpanContext';
 
 use Ref::Util qw/is_plain_hashref/;
-use Types::Standard qw/Object/;
+use Types::Standard qw/Object Str/;
 
 
 
@@ -107,12 +107,52 @@ has client => (
 
 
 
+has default_resource_name => (
+    is          => 'ro',
+    isa         => Str,
+    predicate   => 1,
+);
+
+
+
+has default_service_name => (
+    is          => 'ro',
+    isa         => Str,
+    predicate   => 1,
+);
+
+
+
+has default_service_type => (
+    is          => 'ro',
+    isa         => Str,
+    predicate   => 1,
+);
+
+
+
 sub extract_context {
-    undef
+    my $self = shift;
+    my $carrier_format = shift;
+    
+    my $extracted_data = $self->extract_data_from_carrier( $carrier_format )
+        or return undef;
+    
+    my $trace_id = delete $extracted_data->{ trace_id }
+        or die;
+    my $context = $self
+        ->build_context( $extracted_data )
+        ->with_trace_id( $trace_id );
+    
+    return $context
 }
 
 
 
+sub extract_data_from_carrier { undef };
+
+
+
 sub inject_context { ... }
 
 
@@ -152,15 +192,27 @@ sub build_context {
     my $self = shift;
     my %opts = @_;
     
+    my $resource_name = delete $opts{ resource_name }
+        || $self->default_resource_name;
+    
+    my $service_name  = delete $opts{ service_name }
+        || $self->default_service_name;
+    
+    my $service_type  = delete $opts{ service_type }
+        || $self->default_service_type;
+    
     my $span_context = SpanContext->new(
         
-        resource_name   => $opts{ resource_name },
+        %opts,
+        
+        resource_name   => $resource_name,
         
         maybe
-        service_name    => $opts{ service_name },
+        service_name    => $service_name,
         
         maybe
-        service_type    => $opts{ service_type },
+        service_type    => $service_type,
+        
     );
     
     return $span_context
